#pragma once
#include <fstream>
#include <iostream>
#include <map>
#include <vector>
#include <iomanip>
#include <exception>
#include <algorithm>
#include "AviaInfo.h"

void make_file(){
    std::ofstream out("fileKAMILUPPPPP.bin", std::ios::binary);
    if (!out.is_open()) {
        std::cerr << "File cannot be opened" << std::endl;
        return;
    }
    std::cout << "How much flights do you want to enter?" << std::endl;
    int count;
    std::cin >> count;
    int key = 0;
    AviaInfo info;
    while (count) {
        std::cout << "\t ==== Num " << key+1 << " ====" << std::endl;
        key++;
        std::cout << info << std::endl;
        out.write(reinterpret_cast<char*>(&info), sizeof(AviaInfo));
        count--;
    }
    out.close();
}

void make_map(std::map<int, AviaInfo>& map){
    std::ifstream fin("file.bin", std::ios::binary);
    if (!fin.is_open()) {
        std::cerr << "File cannot be opened" << std::endl;
        return;
    }
    AviaInfo info;
    int key = 0;
    while (fin.read(reinterpret_cast<char*>(&info), sizeof(AviaInfo))) {
        map.insert(std::make_pair(key, info));
        key++;
    }
}

void print_out(){
    std::map<int, AviaInfo> map;
    make_map(map);
    std::ofstream fout("print_out.txt");
    if (!fout.is_open()) {
        std::cerr << "File cannot be opened" << std::endl;
        return;
    }
    fout << "------------------------------------------ Flights --------------------------------------------------------" << std::endl;
    fout << "| Num | Flight | Dep.Date | Dep.Time | Destination |   Surname   | Number Of Luggage | Weight Of Luggage |" << std::endl;
    fout << "-----------------------------------------------------------------------------------------------------------" << std::endl;
    int num = 1;
    for (const auto& elem : map) {
        fout << "|" << std::setw(5) << num << "|"
             << std::setw(8) << elem.second.flight_number << "|"
             << std::setw(10) << elem.second.departure_data << "|"
             << std::setw(10) << elem.second.departure_time << "|"
             << std::setw(13) << elem.second.destination << "|"
             << std::setw(13) << elem.second.surname << "|"
             << std::setw(19) << elem.second.number_of_luggage << "|"
             << std::setw(19) << elem.second.weight_of_luggage << "|"
             << std::endl;
        num++;
    }
    fout.close();
}

void delete_line(int erasing_line)
{
    std::map<int, AviaInfo> map;
    make_map(map);
    map.erase(erasing_line);
    std::ofstream fout("newFile.bin", std::ios::binary);
    if (!fout.is_open()) {
        std::cerr << "File cannot be opened" << std::endl;
        return;
    }
    for (const auto& elem : map)
        fout.write(reinterpret_cast<const char*>(&elem.second), sizeof(AviaInfo));
    fout.close();
}


void search_by_flight_number(const int& p_flight_number){
    std::map<int, AviaInfo> map;
    make_map(map);
    std::vector<AviaInfo> result;

    for (const auto& elem : map) {
        if (elem.second.flight_number == p_flight_number) {
            result.push_back(elem.second);
        }
    }

    std::ofstream fout("new_file_flight_number.bin", std::ios::binary);
    if (!fout.is_open()) {
        std::cerr << "File cannot be opened" << std::endl;
        return;
    }
    for (const auto& elem : result)
        fout.write(reinterpret_cast<const char*>(&elem), sizeof(AviaInfo));
}


void search_by_departure_data(const int& p_departure_data){
    std::map<int, AviaInfo> map;
    make_map(map);
    std::vector<AviaInfo> result;

    for (const auto& elem : map) {
        if (elem.second.departure_data == p_departure_data) {
            result.push_back(elem.second);
        }
    }

    std::ofstream fout("new_file_departure_data.bin", std::ios::binary);
    if (!fout.is_open()) {
        std::cerr << "File cannot be opened" << std::endl;
        return;
    }
    for (const auto& elem : result)
        fout.write(reinterpret_cast<const char*>(&elem), sizeof(AviaInfo));
}


void search_by_weight_of_luggage(const int& normal_weight_of_luggage){
    std::map<int, AviaInfo> map;
    make_map(map);
    std::vector<AviaInfo> result;

    for (const auto& elem : map) {
        if (elem.second.weight_of_luggage > normal_weight_of_luggage) {
            result.push_back(elem.second);
        }
    }

    std::ofstream fout("new_file_heavy_weight_of_luggage.bin", std::ios::binary);
    if (!fout.is_open()) {
        std::cerr << "File cannot be opened" << std::endl;
        return;
    }
    for (const auto& elem : result)
        fout.write(reinterpret_cast<const char*>(&elem), sizeof(AviaInfo));
}

void search_by_destination(const std::string& p_destination){
    std::map<int, AviaInfo> map;
    make_map(map);
    std::vector<AviaInfo> result;

    for (const auto& elem : map) {
        if (strcmp(elem.second.destination, reinterpret_cast<const char*>(&p_destination)) == 0)
            result.push_back(elem.second);
    }

    std::ofstream fout("new_file_destination.bin", std::ios::binary);
    if (!fout.is_open()) {
        std::cerr << "File cannot be opened" << std::endl;
        return;
    }
    for (const auto& elem : result)
        fout.write(reinterpret_cast<const char*>(&elem), sizeof(AviaInfo));
}




